@@ -1,20 +1,19 @@
 """Support session with Muffin framework."""
 
-import sys
 import functools
+import sys
 import typing as t
-from inspect import iscoroutine
+from inspect import iscoroutine, isawaitable
 
-from asgi_sessions import Session
-from asgi_tools.middleware import ASGIApp
-from asgi_tools._types import Receive, Send
 import muffin
+from asgi_sessions import Session
+from asgi_tools._types import Receive, Send
+from asgi_tools.middleware import ASGIApp
 from muffin import ResponseRedirect, Response, Request
 from muffin.plugin import BasePlugin
-from muffin.utils import to_awaitable
 
 
-__version__ = "0.10.8"
+__version__ = "0.10.11"
 __project__ = "muffin-session"
 __author__ = "Kirill Klenov <horneds@gmail.com>"
 __license__ = "MIT"
@@ -23,6 +22,8 @@ __license__ = "MIT"
 SESSION_KEY = 'session'
 USER_KEY = 'user'
 
+F = t.TypeVar('F', bound=t.Callable)
+
 
 class Plugin(BasePlugin):
 
@@ -44,7 +45,7 @@ class Plugin(BasePlugin):
         'login_url': '/login',
     }
 
-    # XXX: Python 3.7
+    # XXX: Python 3.7 (py37)
     if sys.version_info < (3, 8):
         del defaults['cookie_params']['samesite']
 
@@ -57,7 +58,7 @@ class Plugin(BasePlugin):
                 'Use insecure secret key. '
                 'Change SESSION_SECRET_KEY option in your app configuration.')
 
-        self._user_loader = to_awaitable(lambda id_: id_)  # noqa
+        self._user_loader = lambda id_: id_  # noqa
 
         # Install middleware if auto managed
         if self.cfg.auto_manage:
@@ -86,9 +87,9 @@ class Plugin(BasePlugin):
         response.headers['Set-Cookie'] = session.cookie(
             self.cfg.cookie_name, self.cfg.cookie_params)
 
-    def user_loader(self, func):
+    def user_loader(self, func: F) -> F:
         """Register a function as user loader."""
-        self._user_loader = to_awaitable(func)  # noqa
+        self._user_loader = func  # noqa
         return func
 
     async def load_user(self, request):
@@ -98,7 +99,10 @@ class Plugin(BasePlugin):
             if 'id' not in session:
                 return
 
-            request[USER_KEY] = await self._user_loader(session['id'])
+            user = self._user_loader(session['id'])
+            if isawaitable(user):
+                user = await user
+            request[USER_KEY] = user
 
         return request[USER_KEY]
 
